module CURIC::SpinCamera
  def self.spin()
    model = Sketchup.active_model
    view = model.active_view
    spin = Spin.new
    show_dialog_spin(spin)
    view.animation = spin
  end

  def self.show_dialog_spin(spin)
    @spin = spin
    view = Sketchup.active_model.active_view
    camera = view.camera
    position = camera.eye.z.to_mm.round

    center = spin.center
    distance = spin.distance

    step = (distance/100).to_mm.round
    min = (center.z-distance).to_mm.round
    max = (center.z+distance).to_mm.round

    vpheight = view.vpheight
    vpwidth = view.vpwidth
    left = vpwidth-55
    top = vpheight/2 - 320/2

    if @dialog && @dialog.visible?
      return
    end
    
    @dialog = UI::HtmlDialog.new(
    {
      :dialog_title => "Spin camera",
      :preferences_key => "view.spin",
      :scrollable => false,
      :resizable => false,
      :width => 140,
      :height => 370,
      :left => left,
      :top => top,
      :style => UI::HtmlDialog::STYLE_UTILITY
    })

    html = <<-EOT
      <!DOCTYPE html>
      <html>
        <head>
          <meta http-equiv="X-UA-Compatible" content="IE=edge"/>
          <meta charset="UTF-8">
          <style>
            body{
              overflow: hidden;
              margin: 5px;
              font-family: sans-serif;
              font-size: 10pt;
            }
            table{
              width: 100%;
            }
            input[type=range][orient=vertical]
            {
              -webkit-appearance: slider-vertical; /* WebKit */
              width: 8px;
              height: 250px;
              padding: 9px;
              cursor: pointer;
              pointer-events: all;
            }
            #pos_number {
              width: 55px;
            }
          </style>
        <head>
        <body>
          <table>
            <tr>
              <td><span style="padding-left:10px;">Z</span><input id="pos_number" type="text" value=#{position} readonly style="display:none;"/> </td>
              <td>Speed</td>
            </tr>
            <tr>
              <td><input id="position" type="range" orient="vertical" min=#{min} max=#{max} step=#{step} value=#{position} oninput="angleChange()" onchange="angleChange()" /></td>
              <td><input id="speed" type="range" orient="vertical" min=0.1 max=5 step=0.1 value=1 oninput="speedChange()" onchange="speedChange()" /></td>
            </tr>
          </table>

          <div>
            <input  id="play" type="button" value="Pause" onclick="play()"/>
            <input type="button" value="Stop" onclick="stop()"/>
          </div>
          <script>
            function angleChange() {
              var position = document.getElementById("position").value;
              document.getElementById("pos_number").value = position;
              sketchup.position_change(position);
            }

            function speedChange() {
              var angle = document.getElementById("speed").value;
              sketchup.angle_change(angle);
            }

            function stop(){
              document.getElementById("play").value = "Play";
              sketchup.stop()
            }

            function play(){
              var button = document.getElementById("play").value;
              if (button == "Play"){
                sketchup.play()
                document.getElementById("play").value = "Pause";
              }else{
                document.getElementById("play").value = "Play";
                sketchup.pause()
              }
            }
          </script>
        </body>
      </html>
    EOT
    
    @dialog.set_html(html)
    @dialog.add_action_callback("position_change") do |action, position|
      new_position = position.to_f
      @spin.change_position(position.to_l)
    end

    @dialog.add_action_callback("angle_change") do |action, angle|
      angle = angle.to_f
      @spin.angle = angle.degrees
    end

    @dialog.add_action_callback("stop") do |action|
      @spin.stop_animation()
    end

    @dialog.add_action_callback("play") do |action|
      @spin.play_animation()
    end

    @dialog.add_action_callback("pause") do |action|
      @spin.pause_animation()
    end

    @dialog.set_on_closed { @spin.stop_animation() }

    @dialog.show

    @spin.dialog = @dialog

    @dialog
  end

  class Spin
    attr_reader(:center, :distance)
    attr_accessor(:dialog, :angle)

    def initialize
      @model = Sketchup.active_model
      @view = @model.active_view

      @selection = @model.selection

      # stop_animation()
      compute_bound()
      start_animation()

      angle = Sketchup.read_default("curic_view", "spin_angle", 1)
      @angle = angle.degrees

      @pause = false
      @stop = false
    end

    def start_animation
      @selection.remove_observer(@selection_ob) if @selection_ob

      @selection.clear
      @selection_ob = Spin_SelectionObserver.new(self)

      @selection.add_observer(@selection_ob)
    end

    def play_animation
      @pause = false
      @stop = false
      @view.animation = self
      compute_bound()
      start_animation()
    end

    def pause_animation
      @pause = true
      @stop = false
    end

    def stop_animation
      @pause = false
      @stop = true

      @selection.remove_observer(@selection_ob) if @selection_ob
    end

    def stop_animation?
      if @stop
        stop_animation()
        return @stop
      end
    end

    def nextFrame(view)
      if @pause
        return false 
      end

      camera = view.camera
      eye = camera.eye
      vec_center_eye = @center.vector_to(eye)
      up = camera.up
      normal = up

      tr = Geom::Transformation.rotation(@center, Z_AXIS, @angle)
      new_eye = eye.transform(tr)
      new_direction = @center.vector_to(new_eye)
      
      vector = Z_AXIS.cross(new_direction)
      new_up = new_direction.cross(vector)
      view.camera.set(new_eye, @center, new_up)

      view.show_frame

      return !stop_animation?
    end
    
    def compute_eye(view)
      camera = view.camera
      eye = camera.eye
      vec_center_eye = @center.vector_to(eye)
      up = camera.up
      normal = up

      tr = Geom::Transformation.rotation(@center, normal, @angle)
      new_eye = eye.transform(tr)
      new_eye
    end

    def change_position(position)
      camera = @view.camera
      eye = camera.eye

      distance = eye.distance(@center)
      if distance != @distance
        @distance = distance
        refresh_dialog()
        return
      end

      eye.z = position
      vector = @center.vector_to(eye)

      eye = @center.offset(vector, @distance)

      new_direction = vector.reverse
      vector = Z_AXIS.cross(new_direction)
      new_up = new_direction.cross(vector)

      camera.set(eye, @center, new_up)
      nextFrame(@view)
    end

    def selection_change(selection)
      @pause = true
      if @dialog
        @dialog.execute_script('document.getElementById("play").value = "Play";')
      end
    end

    def pause
      # Insert handler code for whatever you need to do when it is paused.
      # p "pause"
    end

    def resume
      # Insert your handler code for whatever you need to do as you resume.
      # p "resume"
    end

    def stop
      # p "stop"
      # p @pause
    end

    def compute_bound
      @bounds = Geom::BoundingBox.new
      @selection.each { |e| @bounds.add(e.bounds) }
      @bounds = @model.bounds if @bounds.empty?

      @center = @bounds.center
      @eye = @view.camera.eye
      @distance = @eye.distance(@center)

      if @dialog
        # set lại z
        position = @eye.z.to_mm.round
        @dialog.execute_script('document.getElementById("pos_number").value = "'+position.to_s+'";')
        @dialog.execute_script('document.getElementById("position").value = "'+position.to_s+'";')
      end
    end

    def refresh_dialog
      if @dialog && @dialog.visible?
        distance = @distance

        step = (distance/100).to_mm.round
        min = (center.z-distance).to_mm.round
        max = (center.z+distance).to_mm.round

        camera = @view.camera
        eye = camera.eye

        position = eye.z.to_mm.round
        @dialog.execute_script('document.getElementById("pos_number").value = "'+position.to_s+'";')
        @dialog.execute_script('document.getElementById("position").value = "'+position.to_s+'";')

        # min, max
        @dialog.execute_script('document.getElementById("position").setAttribute("min", "'+min.to_s+'");')
        @dialog.execute_script('document.getElementById("position").setAttribute("max", "'+max.to_s+'");')
      end
    end
  end

  # Observer
  class Spin_SelectionObserver < Sketchup::SelectionObserver
    def initialize(spin)
      @spin = spin
    end

    def onSelectionBulkChange(selection)
      @spin.selection_change(selection)
    end

    def onSelectionAdded(selection, entity)
      @spin.selection_change(selection)
    end

    def onSelectionCleared(selection)
      @spin.selection_change(selection)
    end
  end
end